'''
api.py

All API calls defined here.

# Tornado API functions template:
class APIHandler(tornado.web.RequestHandler):
    def get(self):
        #get the Argument that User had passed as name in the get request
        userInput=self.get_argument('name')
        welcomeString=sayHello(userInput)
        #return this as JSON
        self.write(json.dumps(welcomeString))

    def post(self):
        user = self.get_argument("username")
        passwd = self.get_argument("password")
        time.sleep(10)
        self.write("Your username is %s and password is %s" % (user, passwd))
'''

class loadJsonsList(tornado.web.RequestHandler):
    def get(self):
        # /API/loadJsonsList
        start = time.time()
        returnContent = recursiveDropdown(routesFolder,ext='.json')
        self.write(returnContent)
        end = time.time()
        logmessage("loadJsonsList GET call took {} seconds.".format(round(end-start,2)))

class loadJson(tornado.web.RequestHandler):
    def get(self):
        # /API/loadJson?route=filename
        start = time.time()
        filename = self.get_argument('route',default='')
        logmessage(filename)
        if not filename or not os.path.exists(os.path.join(routesFolder,filename)):
            self.set_status(400)
            self.write("Error: invalid filename.")
            return
        
        with open(os.path.join(routesFolder,filename)) as fh:
            a = json.load(fh)
        self.write(json.dumps(a) )
        end = time.time()
        logmessage("loadJson GET call took {} seconds.".format(round(end-start,2)))

    def post(self):
        # /API/loadJson?route=filename&pw=password
        start = time.time()
        pw = self.get_argument('pw',default='')
        if pw != configRules.get('PW_MAPPER',''):
            self.set_status(400)
            self.write("Error: Need Mapper level password.")
            return
        filename = self.get_argument('route',default='')
        data = json.loads( self.request.body.decode('UTF-8'), object_pairs_hook=OrderedDict )
        logmessage(filename)

        saveStatus = saveRoute(filename,data)
        self.write(saveStatus)
        end = time.time()
        logmessage("loadJson POST call (to save a route) took {} seconds.".format(round(end-start,2)))

class GPXUpload(tornado.web.RequestHandler):
    def post(self):
        # /API/GPXUpload
        start = time.time()

        gpx_string = self.request.files['gpx'][0]['body']
        gpxDict = xmltodict.parse(gpx_string, attr_prefix='')
        
        waypointDF1 = pd.DataFrame(gpxDict['gpx']['wpt'])
        waypointDF2 = waypointDF1[waypointDF1.name.str.lower() != 'bus stop'][['name','lat','lon','time']].copy()
        # removing extra bus stop entries
        waypointDict = waypointDF2.to_dict(orient='records', into=OrderedDict)
        
        trackDict = [ [ x['lat'], x['lon'] ] for x in gpxDict['gpx']['trk']['trkseg']['trkpt'] ]

        returnJson = json.dumps({'waypoint':waypointDict, 'track':trackDict}, ensure_ascii=False)
        self.write(returnJson)
        end = time.time()
        logmessage("GPXUpload POST call took {} seconds.".format(round(end-start,2)))


class routeSuggest(tornado.web.RequestHandler):
    def get(self):
        # /API/routeSuggest?route=filename
        start = time.time()

        filename = self.get_argument('route',default='')
        if not len(filename):
            self.set_status(400)
            self.write("Error: Load a valid route.")
            return
        
        logmessage('routeSuggest:',filename)
        status = routeSuggestFunc(filename)
        self.write(status)
        end = time.time()
        logmessage("routeSuggest GET call took {} seconds.".format(round(end-start,2)))

class routeLock(tornado.web.RequestHandler):
    def get(self):
        # /API/routeLock?route=filename&pw=pw
        start = time.time()

        pw = self.get_argument('pw',default='')
        if pw != configRules.get('PW_ADMIN',''):
            self.set_status(400)
            self.write("Error: Need Admin level password.")
            return

        filename = self.get_argument('route',default='')
        if not len(filename):
            self.set_status(400)
            self.write("Error: Load a valid route.")
            return
        
        logmessage('routeLock:',filename)
        status = routeLockFunc(filename)
        self.write(status)
        end = time.time()
        logmessage("routeLock GET call took {} seconds.".format(round(end-start,2)))

#################################
'''
discard; load csv on js side using papa.parse

class getDatabank(tornado.web.RequestHandler):
    def get(self):
        # /API/getDatabank?searchName=${searchName}&fuzzyFlag=${fuzzyFlag}&jellyFlag=${jellyFlag}
        start = time.time()
        searchName = self.get_argument('searchName',default='')
        logmessage(searchName)
        fuzzyFlag = self.get_argument('fuzzyFlag',default='n')
        jellyFlag = self.get_argument('jellyFlag',default='n')

        with open(configFile) as f:
            configDict = json.load(f)
        databank = configDict.get('databank', 'hydstopsGoogle3.csv')

        try:
            df = pd.read_csv(databankFolder+databank)
        except OSError as e:
            logmessage('Error:',e)
            self.set_status(400)
            self.write('Error:' + str(e))
            return

        if jellyFlag == 'y':
            Jstart = time.time()
            df2 = jellyPanda(searchName,df)
            Jend = time.time()
            logmessage("Jellyfish scoring took {} seconds.".format(round(Jend-Jstart,3)))    
        else: df2 = df
        
        if fuzzyFlag == 'y':
            Fstart = time.time()
            df3 = fuzzyPanda(searchName,df2)
            Fend = time.time()
            logmessage("Fuzywuzzy scoring took {} seconds.".format(round(Fend-Fstart,3)))    
        else: df3 = df2
        
        rightJson = df3.to_json(orient='records', force_ascii=False)
        
        del df,df2,df3 # cleaning up
        
        self.write(rightJson)
        end = time.time()
        logmessage("getDatabank GET call took {} seconds.".format(round(end-start,2)))
'''
