def logmessage( *content ):
    timestamp = '{:%Y-%b-%d %H:%M:%S} :'.format(datetime.datetime.now())
    # from https://stackoverflow.com/a/26455617/4355695
    line = ' '.join(str(x) for x in list(content))
    # str(x) for x in list(content) : handles numbers in the list, converts them to string before concatenating. 
    # from https://stackoverflow.com/a/3590168/4355695
    print(line) # print to screen also
    f = open(logFolder + 'log.txt', 'a', newline='\r\n', encoding='utf8') #open in append mode
    print(timestamp, line, file=f)
    # `,file=f` argument at end writes the line, with newline as defined, to the file instead of to screen. 
    # from https://stackoverflow.com/a/2918367/4355695
    f.close()


def recursiveDropdown(targetFolder = '.', ext=''):
    '''
    called from loadJsonsList API call
    recursiveDropdown(targetFolder = '.', ext='')
    recursively scans target folder for all files ending with extension ext (or all files by default), and generates innerHTML for a dropdown menu, with respective sub-folder paths as optgroups.
    '''
    if targetFolder.endswith('/'): targetFolder = targetFolder[:-1]
    content = '<option value="">Select one</option>'

    # traverse through all files, folders under a path recursively. from https://stackoverflow.com/a/2212698/4355695
    for root, subdirs, files in os.walk(targetFolder):
        folder = root[len(targetFolder)+1:]
        targetFiles = [x for x in files if x.lower().endswith(ext)]
        targetFiles.sort()
        
        if len(folder) and len(targetFiles):
            content += '<optgroup label="{}">'.format(folder)
        for file in targetFiles:
            filepath = os.path.join(root,file)[len(targetFolder)+1:]
            content += '<option value="{}">{}</option>'.format(filepath,file)
        if len(folder) and len(targetFiles):
            content += '</optgroup>'
    return content

def saveRoute(filename,data=[]):
    if not os.path.exists(os.path.join(routesFolder,filename)):
        return 'Hey! this file {} doesnt even exist!'.format(filename)
    
    routeD = json.load(open(os.path.join(routesFolder,filename)), object_pairs_hook=OrderedDict)
    
    df = pd.DataFrame(data, dtype=str).fillna('')

    df0 = df[ (df['direction_id'] == '0') | (df['direction_id'] == '')].copy().reset_index(drop=True)
    df1 = df[ (df['direction_id'] == '1')].copy().reset_index(drop=True)

    # again, which one to loop through? 
    # How will the suggested stops come? Ok, let's embed them into the json array itself as another column, "suggested".
    # Structure of suggested: [source,lat,lon]; 
    # # ex: src5,34.45,122.54;osm,34.45,122.54
    # They'll get interpreted by the JS.
    # So all the data is with the df. We do a full replace of the stops arrays in the json.
    # note: we're not sorting here, those sequence numbers are just for show. taking stops in the order they're coming in.
    
    routeD['stopsArray0'] = [] #reset!
    for n,row in df0.iterrows():
        stopRow = OrderedDict({
            'stop_sequence': n+1,
            'stop_name' : row['stop_name'] if 'stop_name' in row else '',
            'stop_lat' : row['stop_lat'] if 'stop_lat' in row else '',
            'stop_lon' : row['stop_lon'] if 'stop_lon' in row else '',
            'stop_desc' : row['stop_desc'] if 'stop_desc' in row else '',
            'offset' : row['offset'] if 'offset' in row else '',
            'confidence' : row['confidence'] if 'confidence' in row else '',
            'suggested' : row['suggested'] if 'suggested' in row else ''
        })
        routeD['stopsArray0'].append(stopRow.copy())

    routeD['stopsArray1'] = [] #reset!
    for n,row in df1.iterrows():
        stopRow = OrderedDict({
            'stop_sequence': n+1,
            'stop_name' : row['stop_name'] if 'stop_name' in row else '',
            'stop_lat' : row['stop_lat'] if 'stop_lat' in row else '',
            'stop_lon' : row['stop_lon'] if 'stop_lon' in row else '',
            'stop_desc' : row['stop_desc'] if 'stop_desc' in row else '',
            'offset' : row['offset'] if 'offset' in row else '',
            'confidence' : row['confidence'] if 'confidence' in row else '',
            'suggested' : row['suggested'] if 'suggested' in row else ''
        })
        routeD['stopsArray1'].append(stopRow.copy())

    json.dump(routeD, open(os.path.join(routesFolder,filename), 'w'), indent=2)
    # print(routeD)
    # print()
    # print(df0)

    return 'Saved '+filename

def routeSuggestFunc(filename):
    if not os.path.exists(os.path.join(routesFolder,filename)):
        logmessage('routeSuggestFunc: this file {} doesnt even exist! Exiting'.format(filename))
        return 'Hey! this file {} doesnt even exist!'.format(filename)
    
    routeD = json.load(open(os.path.join(routesFolder,filename)), object_pairs_hook=OrderedDict)
    
    for stopRow in routeD.get('stopsArray0',[]):
        stopRow['suggested'] = findNameSuggestions(stopRow.get('stop_name',''),direction_id=0)
    
    for stopRow in routeD.get('stopsArray1',[]):
        stopRow['suggested'] = findNameSuggestions(stopRow.get('stop_name',''),direction_id=1)
    
    json.dump(routeD, open(os.path.join(routesFolder,filename), 'w'), indent=2)  
    return 'Added suggested locations for stop names in ' +filename


def findNameSuggestions(stop_name,direction_id=1):
    
    return 'test,20.0,20.0'

def routeLockFunc(filename):
    return 'Under Construction!'