/* main.js */

//#####################################
/* constants */
const APIpath = 'API/';
const waitTime = 15;
const version = 'v2.0';

//#####################################
/* global variables */
var stops0 = ace.edit("stops0");
var stops1 = ace.edit("stops1");
var globaltest;
var showHints = true;
var statusTimer; // clear out any previous timer in the status messages. from https://stackoverflow.com/a/3015351/4355695

//#####################################
// On page load
$( function() { // run  when document ready

	$('#version').html(version);
	// bootstrap popover
	$('[data-toggle="popover"]').popover();
	$('[data-toggle="tooltip"]').tooltip();
	
	loadRoutesList();
	loadJsonsList();
	// drowndown handlers:

	$('#routeSelect').on('change', function (e) {
		var valueSelected = this.value;
		console.log(valueSelected);
		if( valueSelected == '') { 
			return;
		}
		loadRoute(valueSelected);
	});

	$('#jsonSelect').on('change', function (e) {
		var valueSelected = this.value;
		console.log(valueSelected);
		if( valueSelected == '') { 
			return;
		}
		loadJson(valueSelected);
	});

	// Clone routeName to routeFileName
	$("#routeName").bind("change keyup", function(){
		$("#routeName").val( this.value.toUpperCase() );
		$("#routeFileName").val( this.value.replace(/[^A-Za-z0-9-_]/g, "_") );
	});

	// Automatically turn hints off after this many seconds, if they're still on.
	setTimeout(function() { if(showHints) toggleHints(); }, 60*1000);

	// suppress browser from scrolling to top
	// from https://stackoverflow.com/questions/1601933/how-do-i-stop-a-web-page-from-scrolling-to-the-top-when-a-link-is-clicked-that-t
	$('a').click(function(e) {
		//doSomething();
		return false;
	});
});

//#####################################
// API Call functions

function loadRoutesList() {
	$.get( `${APIpath}loadRoutesList`, function( data ) {
		//console.log(data);
		$('#routeSelect').html(data);
		$('#routeSelect').trigger('chosen:updated'); 
		$('#routeSelect').chosen({disable_search_threshold: 1, search_contains:true, width:200, height: 400, placeholder_text_single:'Pick a route'});
	});
}

function loadRoute(route_id) {
	if(route_id == '') return;

	clearEverything('loading..');
	/*
	document.getElementById('timings0').value = document.getElementById('timings1').value = document.getElementById('routeName').value = 'loading...';
	stops0.setValue('loading...');
	stops1.setValue('loading...');
	*/

	//$('#routeName').val('loading..');
	$.get( `${APIpath}loadRouteCSV?route=${route_id}`, function( data ) {
		console.log('loadRouteCSV GET call successful for',route_id);
		jsondata = JSON.parse(data);
		//console.log(jsondata);

		// just for psychological satisfaction, wait another half a second before setting all the values
		setTimeout(function() { 
			;
		}, 0.5*1000);

		clearEverything();
		
		$('#routeName').val(jsondata['route']);
		$('#routeFileName').val(jsondata['filename'].toUpperCase());

		stops0.setValue(jsondata['stops0']);
		stops0.clearSelection();
		stops1.setValue(jsondata['stops1']);
		stops1.clearSelection();

		$('#timings0').val(jsondata['timings0']);
		$('#timings1').val(jsondata['timings1']);

		/* oh crap this is CSV loading : the only data loading here is : sequence of stops, default timings if any
		$('#timingsWeekend0').val(jsondata['timingsWeekend0']);
		$('#timingsWeekend1').val(jsondata['timingsWeekend1']);
		$('#extra0').val(jsondata['extra0']);
		$('#extra1').val(jsondata['extra1']);
		$('#freq0').val(jsondata['freq0']);
		$('#freq1').val(jsondata['freq1']);
		*/
		

	});
}

function saveRoute() {
	
	var data = {};
	data.routeFileName = document.getElementById('routeFileName').value;
	if(data.routeFileName == '') {
		alert('Please put a proper filename for the route, in green box right side. Right now NOTHING is saved!');
		$('.saveStatus').html('Invalid filename given.');
		clearTimeout(statusTimer);
		statusTimer = setTimeout(function() { $('.saveStatus').html(''); }, waitTime/2*1000);
		return;
	}
	$('.saveStatus').html(`<b>Saving to routes/${data.routeFileName}.json ...</b>`);

	// lets get it all together!
	// route-meta:
	data.routeName = document.getElementById('routeName').value;
	data.routeLongName = document.getElementById('routeLongName').value;
	data.depot = document.getElementById('depot').value;
	data.busType = document.getElementById('busType').value;
	// stops:
	data.stops0 = stops0.getValue();
	data.stops1 = stops1.getValue();
	// timings:
	data.timings0 = document.getElementById('timings0').value;
	data.timings1 = document.getElementById('timings1').value;
	data.timingsWeekend0 = document.getElementById('timingsWeekend0').value;
	data.timingsWeekend1 = document.getElementById('timingsWeekend1').value;
	// frequency:
	data.freq0 = document.getElementById('freq0').value;
	data.freq1 = document.getElementById('freq1').value;
	// extra:
	data.extra0 = document.getElementById('extra0').value;
	data.extra1 = document.getElementById('extra1').value;
	

	
	console.log(data);

	request = $.ajax({
		url: `${APIpath}saveRoute`,
		type: "post",
		data: JSON.stringify(data),
		success: function(result){
			console.log('saveRoute POST call successful.');
			//console.log('result:',result);
			loadJsonsList();
			

			setTimeout(function() { 
				// Just for psychological satisfaction, even when all is done, wait a second before clearing out and telling user that all is saved.
				$('.saveStatus').html(`<b>Saved ${data.routeFileName}.json in routes folder.</b><br>It should now show up in the "Review saved routes" dropdown.`);
				clearTimeout(statusTimer);
				statusTimer = setTimeout(function() { $('.saveStatus').html(''); }, waitTime*1000);
			}, 1*1000);
			
		}
	});
	
}

function loadJsonsList() {
    $.get( `${APIpath}loadJsonsList`, function( data ) {
        console.log('GET request loadJsonsList successful.');
        $('#jsonSelect').html(data);
        $('#jsonSelect').trigger('chosen:updated'); 
        $('#jsonSelect').chosen({disable_search_threshold: 1, search_contains:true, width:200, placeholder_text_single:'Pick a route'});
    });
}

function loadJson(route_id) {
	if(route_id == '') return;

	clearEverything(loaderText='loading...');
	
	//$('#routeName').val('loading..');
	$.get( `${APIpath}loadJson?route=${route_id}`, function( data ) {
		console.log('loadJson GET call successful for',route_id);
		jsondata = JSON.parse(data);
		//console.log(jsondata);
		globaltest = jsondata;
		clearEverything();

		$('#routeName').val(jsondata['routeName']);
		$('#routeFileName').val(jsondata['routeFileName']);
		$('#routeLongName').val(jsondata['routeLongName']);
		$('#depot').val(jsondata['depot']);
		$('#busType').val(jsondata['busType']);
		$('#timings0').val(jsondata['timings0']);
		$('#timings1').val(jsondata['timings1']);
		$('#timingsWeekend0').val(jsondata['timingsWeekend0']);
		$('#timingsWeekend1').val(jsondata['timingsWeekend1']);
		$('#extra0').val(jsondata['extra0']);
		$('#extra1').val(jsondata['extra1']);
		$('#freq0').val(jsondata['freq0']);
		$('#freq1').val(jsondata['freq1']);

		// 8.12.18: Intervention : Check if stopsArray keys are present and having length:
		if( jsondata['stopsArray0'] instanceof Array) {
			var stopsText = '';
			jsondata['stopsArray0'].forEach( function(x) {
				stopsText += x['stop_name'];
				if( x['stop_id']) stopsText += ',' + x['stop_id'];
				if( x['stop_lat']) stopsText += ',' + x['stop_lat'];
				if( x['stop_lon']) stopsText += ',' + x['stop_id'];
				stopsText += '\n';
			});
			stops0.setValue(stopsText);
			console.log('loaded stops from json array instead of string');
		}
		else stops0.setValue(jsondata['stops0']);
		
		if( jsondata['stopsArray1'] instanceof Array) {
			var stopsText = '';
			jsondata['stopsArray1'].forEach( function(x) {
				stopsText += x['stop_name'];
				if( x['stop_id']) stopsText += ',' + x['stop_id'];
				if( x['stop_lat']) stopsText += ',' + x['stop_lat'];
				if( x['stop_lon']) stopsText += ',' + x['stop_lon'];
				stopsText += '\n';
			});
			stops1.setValue(stopsText);
		}
		else stops1.setValue(jsondata['stops1']);
		
		// make the ace editor boxes unselected; by default they're select-all when just loaded.
		stops0.clearSelection();
		stops1.clearSelection();
	});
}
//#####################################
// Other Functions

function toggleHints() {
	if(showHints) {
		// using a brute force all-three-hammers way after using each singly just failed. Only doing 'dispose' caused bad runtime errors on the page to build up. Reference: https://github.com/twbs/bootstrap/issues/475
		$('[data-toggle="popover"]').popover('hide');
		$('[data-toggle="popover"]').popover('disable');
		$('[data-toggle="popover"]').popover('dispose');

		$('[data-toggle="tooltip"]').tooltip('hide');
		$('[data-toggle="tooltip"]').tooltip('disable');
		$('[data-toggle="tooltip"]').tooltip('dispose');

		showHints = false;
		$('#toggleHintsLink').html('Hints: OFF');
	} else {
		$('[data-toggle="popover"]').popover();
		$('[data-toggle="tooltip"]').tooltip();
		showHints = true;
		$('#toggleHintsLink').html('Hints: ON');
	}
}

function reverse() {
	let sequence = stops0.getValue().split('\n');
	sequence.reverse();
	stops1.setValue(sequence.join('\n'));
	stops1.clearSelection();
}

function clearEverything(loaderText='') {
	$(':input').val(loaderText);
	stops0.setValue(loaderText);
	stops1.setValue(loaderText);
	$('.saveStatus').html('');
}

function weekendCopy(dir=0) {
	$('#timingsWeekend'+dir).val($('#timings'+dir).val());
	
}
