/* routemap.js
moved here from RouteMapper
*/

// ######################################
/* 1. GLOBAL VARIABLES */

const databankColor = '#dde88f';
const stopOnwardColor = 'green'
const stopReturnColor = 'blue';
var globalIndex = '';
var clickedflag = false;
var debugMode = true;
var URLParams = {}; // for holding URL parameters
var firstRunDone = false; // for preventing 3-time calling of mapStops() on startup.
var mappedList = [];
// Map: these layers need to have global scope, and are mentioned in tabulator constructors, hence need to be defined here.
var stopsLayer = new L.geoJson(null);
var databank = new L.geoJson(null);
var lineLayer = new L.geoJson(null);
// initiate GPX layers in advance
var trackLayer = new L.geoJson(null);
var waypointLayer = new L.geoJson(null);

var globalRoute = '';
// #################################
/* 2. Initiate tabulators */

// first, define custom functions that will be called by the 
var stopsTotal = function(values, data, calcParams){
	var calc = values.length;
	return calc + ' stops total';
}

//########################
// now the actual table construction

const routeTable_height = "400px";
var routeTable = new Tabulator("#routeTable", {
	height:routeTable_height,
	selectable:1, // make max 1 row click-select-able. http://tabulator.info/docs/3.4?#selectable
	movableRows: true, //enable user movable rows
	//layout:"fitColumns", //fit columns to width of table (optional)
	//index: "orig_id", 
	layout:"fitDataFill",
	addRowPos: "top",
	tooltipsHeader:true,
	dataTree:true, 
	dataTreeCollapseElement: "<span class='badge badge-secondary'>-</span> ",
	dataTreeExpandElement: "<span class='badge badge-secondary'>+</span> ",

	columns:[ //Define Table Columns
		{rowHandle:true, formatter:"handle", headerSort:false, frozen:true, width:30, width:15},
		//{title:"orig_id", field:"orig_id", frozen:true, headerFilter:"input" },
		//{title:"Num", width:40, formatter: "rownum", headerSort:false}, // row numbering
		{title:"dir", field:"direction_id", editor:"input", headerFilter:"input", frozen:true, headerTooltip:"direction: 0 for onward, 1, for return", width:15, headerSort:false },
		{title:"seq", field:"stop_sequence", editor:"input", headerFilter:"input", frozen:true, headerTooltip:"stop sequence number", width:15, headerSort:false },
		{title:"stop_name", field:"stop_name", editor:"input", headerFilter:"input", bottomCalc:stopsTotal, width:100, headerSort:false },
		{title:"stop_lat", field:"stop_lat", editor:"input", headerSort:false, width:60, headerTooltip:"latitude" },
		{title:"stop_lon", field:"stop_lon", editor:"input", headerSort:false, width:60, headerTooltip:"longitude" },
		{title:"offset", field:"offset", headerFilter:"input", editor:"input", headerTooltip:"arrival time, in mins after first stop", headerSort:false },
		{formatter:"buttonCross", align:"center", title:"del", headerSort:false, headerTooltip:"delete a stop", cellClick:function(e, cell){
			if(confirm('Are you sure you want to delete this entry?'))
				cell.getRow().delete();
			}},
		{title:"confidence", field:"confidence", headerFilter:"input", width:100, headerSort:false, editor:"select", editorParams:confidence_options },
		{title:"stop_desc", field:"stop_desc", editor:"input", headerFilter:"input", width:100, headerSort:false },
		{title:"suggested", field:"suggested", editor:"input", headerFilter:"input", width:50, headerSort:false },
		//{title:"source", field:"source", editor:"input", headerFilter:"input", width:100, headerSort:false },
		//{title:"source_stop_id", field:"source_stop_id", editor:"input", headerFilter:"input", width:100, headerSort:false },
		//{title:"source_stop_name", field:"source_stop_name", editor:"input", headerFilter:"input", width:100, headerSort:false },
		//{title:"time", field:"time", headerFilter:"input", editor:"input", headerTooltip:"first trip's timing" },
	],
	rowSelected:function(row){ //when a row is selected
		// document.getElementById('searchName').value = row.getData().stop_name; // of fuzzy, retired
		// console.log('ID: ' + row.getIndex());
		// globalIndex = row.getIndex();
		
		setTimeout(function() {
			hide( document.querySelector("#panel") ); // when changing to next stop, old stop shouldn't show.

			// if it has lat-long, show on map!
			// if (map.hasLayer(databank)) map.removeLayer(databank); // remove databank layer of past stop's matches when switching to another row.
			var lat = parseFloat(row.getData().stop_lat);
			var lon = parseFloat(row.getData().stop_lon);
			if ( checklatlng(lat,lon) ) {
				if (!map.hasLayer(stopsLayer)) map.addLayer(stopsLayer); // to do : check if visible already, then show if not
				mapZoomHere(lat,lon,16);
			} 
			// else loadRight(); // if not already mapped, then fuzzy search
		}, 500); //wait then run all this
	},
	rowDeselected:function(row){ //when a row is selected
		hide( document.querySelector("#panel") );
	},
	dataEdited:function(data){
		//if(data.length) mapStops();
	},
	rowUpdated:function(row){
		setTimeout(mapStops,1000);
	},
	dataFiltered:function(filters, rows){
		if(rows.length) { 
			setTimeout(function() {
				mapStops(); 
				if(lineLayer.getLayers().length && map.hasLayer(lineLayer)) {
					map.removeLayer(lineLayer);
					routeLines();
				}
			},1000);
		}
	},
	dataLoaded:function(data){
		if(!data.length) return;
		setTimeout(function() { mapStops(firstRun=true); },1000); // load the map but after an interval

		/*
		//empty the select element
		olcSelectTop.empty();
		//fill the select element with options from the data
		olcSelectTop.append("<option value=''>All</option>");
		ol6Collector = new Set(); // using sets in js. from https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Set
		data.forEach(function(item){
			if(!item.olc6) return;
			if( item.olc6.length > 0 && !ol6Collector.has(item.olc6) ) {
				value = item.olc6.slice(start=3, end=7);
				ol6Collector.add(item.olc6);
				olcSelectTop.append("<option value='" + item.olc6 + "'>" + value + "</option>");
			}
		});
		*/
	},

	cellEditing:function(cell){
		// pop up the stop on the map when user starts editing
		row = cell.getRow(); //.getData().stop_id);
		row.toggleSelect();
	},
});

const databankTable_height = 400;
var databankTable = new Tabulator("#databankTable", {
	height: databankTable_height,
	selectable:1, // make max 1 row click-select-able. http://tabulator.info/docs/3.4?#selectable
	movableRows: true, //enable user movable rows
	movableColumns: true, //enable user movable columns
	//layout:"fitColumns", //fit columns to width of table (optional)
	index: "stop_id", 
	layout:"fitDataFill",
	addRowPos: "top",
	tooltipsHeader:true, //enable header tooltips
	columns:[ //Define Table Columns
		// stop_id,stop_name,stop_lat,stop_lon,zone_id,wheelchair_boarding
		{rowHandle:true, formatter:"handle", headerSort:false, frozen:true, width:30, minWidth:30},
		//{title:"Num", width:40, formatter: "rownum", frozen:true }, // row numbering
		
		{title:"sr", field:"sr", headerFilter:"input", frozen:true },
		{title:"stop_name", field:"stop_name", headerFilter:"input", bottomCalc:stopsTotal, frozen:true },
		{title:"stop_lat", field:"stop_lat", headerSort:false },
		{title:"stop_lon", field:"stop_lon", headerSort:false },
		{title:"rank", field:"rank", headerFilter:"input" },
		{title:"source", field:"source", headerFilter:"input" },
		{title:"stop_id", field:"stop_id", headerFilter:"input" },
		{title:"stop_desc", field:"stop_desc", headerFilter:"input" },

		/*
		// Top algos, one of Fuzzywuzy and other of Jellyfish
		{title:"Fpartial", field:"Fpartial", headerFilter:"input", sorter:"number", headerTooltip:"Fuzzywuzzy partial ratio" },
		{title:"JjaroW", field:"JjaroW", headerFilter:"input", sorter:"number", headerTooltip:"Jellyfish jaro_winkler" },

		// Fuzzywuzzy:
		
		{title:"Fsimple", field:"Fsimple", headerFilter:"input", sorter:"number", headerTooltip:"Fuzzywuzzy ratio" },
		{title:"FtokenSort", field:"FtokenSort", headerFilter:"input", sorter:"number", headerTooltip:"Fuzzywuzzy token_sort_ratio" },
		{title:"FtokenSet", field:"FtokenSet", headerFilter:"input", sorter:"number", headerTooltip:"Fuzzywuzzy token_set_ratio" },
		
		// Jellyfish:
		{title:"Jlevenshtein", field:"Jlevenshtein", headerFilter:"input", sorter:"number", headerTooltip:"Jellyfish levenshtein_distance" },
		{title:"Jdamerau", field:"Jdamerau", headerFilter:"input", sorter:"number", headerTooltip:"Jellyfish damerau_levenshtein_distance" },
		{title:"Jhamming", field:"Jhamming", headerFilter:"input", sorter:"number", headerTooltip:"Jellyfish hamming_distance" },
		{title:"JjaroD", field:"JjaroD", headerFilter:"input", sorter:"number", headerTooltip:"Jellyfish jaro_distance" },
		{title:"Jmatch", field:"Jmatch", headerFilter:"input", sorter:"number", headerTooltip:"Jellyfish match_rating_comparison" },
		*/
	],
	rowSelected:function(row){ //when a row is selected
		var lat = parseFloat(row.getData().stop_lat);
		var lon = parseFloat(row.getData().stop_lon);
		var otherdata = `${row.getData().stop_name}<br>ID:${row.getData().stop_id}, ${row.getData().source}`;
		mapZoomHere(lat,lon); 

		askToMapSelected(lat,lon,otherdata); // ask user on the map if they want to map this
		dragmarker.remove(); clickedflag=false;

	},
	rowDeselected:function(row){ //when a row is selected
		hide( document.querySelector("#panel") );
	},
	dataLoaded: function(data) {
		if(!data.length) return;

		/*
		//empty the select element
		olcSelectBottom.empty();
		//fill the select element with options from the data
		olcSelectBottom.append("<option value=''>All</option>");
		ol6Collector = new Set(); // using sets in js. from https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Set
		data.forEach(function(item){
			if(!item.olc6) return;
			if( item.olc6.length > 0 && !ol6Collector.has(item.olc6) ) {
				value = item.olc6.slice(start=3, end=7);
				ol6Collector.add(item.olc6);
				olcSelectBottom.append("<option value='" + item.olc6 + "'>" + value + "</option>");
			}	
		});
		*/
	},

	dataSorted:function(sorters, rows){
		// first its loaded then its sorted and then the mapping happens on the sorted data.
		if(rows.length) setTimeout(function() {
			limit = $('#databankLimit').val();
			mapDataBank(limit=limit);
			mapStops(); // putting them on top
		}, 1000);
	}

});

// #################################
/* 3. Initiate map */

var osmLink = '<a href="http://openstreetmap.org">OpenStreetMap</a>';
var MBAttrib = '&copy; ' + osmLink + ' Contributors & <a href="https://www.mapbox.com/about/maps/">Mapbox</a>';
var mapboxUrl = 'https://{s}.tiles.mapbox.com/v3/{id}/{z}/{x}/{y}.png';
var scenicUrl = 'https://api.mapbox.com/styles/v1/nikhilsheth/cj8rdd7wu45nl2sps9teusbbr/tiles/256/{z}/{x}/{y}?access_token=pk.eyJ1IjoibmlraGlsc2hldGgiLCJhIjoiQTREVlJuOCJ9.YpMpVVbkxOFZW-bEq1_LIw' ; 

var MBstreets = L.tileLayer(mapboxUrl, {id: 'nikhilsheth.m0mlpl2d', attribution: MBAttrib, maxZoom: 20}),
	MBsatlabel = L.tileLayer(mapboxUrl, {id: 'nikhilsheth.m0mmaa87', attribution: MBAttrib, maxZoom: 20}),
	MBsat = L.tileLayer(mapboxUrl, {id: 'nikhilsheth.m0mni8e7', attribution: MBAttrib, maxZoom: 20}),
	MBlight = L.tileLayer(mapboxUrl, {id: 'nikhilsheth.m0mmobne', attribution: MBAttrib, maxZoom: 20}),
	MBdark = L.tileLayer(mapboxUrl, {id: 'nikhilsheth.jme9hi44', attribution: MBAttrib, maxZoom: 20}),
	OsmIndia = L.tileLayer(mapboxUrl, {id: 'openstreetmap.1b68f018', attribution: MBAttrib, maxZoom: 20}),
	GithubLight = L.tileLayer('http://{s}.tiles.mapbox.com/v3/github.map-xgq2svrz/{z}/{x}/{y}.png', {attribution: MBAttrib, maxZoom: 20}),
	scenic = L.tileLayer('https://api.mapbox.com/styles/v1/nikhilsheth/cj8rdd7wu45nl2sps9teusbbr/tiles/256/{z}/{x}/{y}?access_token=pk.eyJ1IjoibmlraGlsc2hldGgiLCJhIjoiQTREVlJuOCJ9.YpMpVVbkxOFZW-bEq1_LIw', {attribution: MBAttrib, maxZoom: 20}) ; 
var gStreets = L.tileLayer('http://{s}.google.com/vt/lyrs=m&x={x}&y={y}&z={z}',{
		maxZoom: 20,
		subdomains:['mt0','mt1','mt2','mt3']
});
	
var gHybrid = L.tileLayer('http://{s}.google.com/vt/lyrs=s,h&x={x}&y={y}&z={z}',{
		maxZoom: 20,
		subdomains:['mt0','mt1','mt2','mt3']
});
var gSat = L.tileLayer('http://{s}.google.com/vt/lyrs=s&x={x}&y={y}&z={z}',{
    maxZoom: 20,
    subdomains:['mt0','mt1','mt2','mt3']
});

var map = new L.Map('map', {
	center: STARTLOCATON,
	zoom: 10,
	layers: [gStreets],
	scrollWheelZoom: true,
	maxZoom: 20
});

$('.leaflet-container').css('cursor','crosshair'); // from https://stackoverflow.com/a/28724847/4355695 Changing mouse cursor to crosshairs

L.control.scale({metric:true, imperial:false}).addTo(map);

var baseLayers = {
	"Scenic" : scenic,
	"OpenStreetMap.IN": OsmIndia,
	"Streets": MBstreets,
	"Satellite": MBsatlabel ,
	//"Mapbox Light": MBlight,
	"Dark" : MBdark,
	"gStreets": gStreets,
	"gHybrid": gHybrid,
	"gSat": gSat
};


// SVG renderer
var myRenderer = L.canvas({ padding: 0.5 });

// Marker for positioning new stop or changing location of stop
var dragmarkerOptions = {
	//renderer: myRenderer,
	radius: 5,
	fillColor: "red",
	color: null,
	weight: 1,
	opacity: 1,
	fillOpacity: 0.8,
	interactive: false
};
var dragmarker = L.circleMarker(null, dragmarkerOptions);
//layerControl.addOverlay(dragmarker , "Marker");

//lineLayer.addTo(map);

var overlays = {
	"Marker" : dragmarker,
	"data bank": databank,
	"route lines": lineLayer,
	"mapped stops": stopsLayer,
	"GPX waypoints": waypointLayer,
	"GPX track": trackLayer
};
var layerControl = L.control.layers(baseLayers, overlays, {collapsed: true, autoZIndex:false}).addTo(map); 

// minimap:
//var miniMe = new L.TileLayer(mapboxUrl, {minZoom: 0, maxZoom: 18, attribution: MBAttrib, id: 'nikhilsheth.m0mlpl2d' });

var miniMe = L.tileLayer('http://{s}.google.com/vt/lyrs=m&x={x}&y={y}&z={z}',{
		maxZoom: 20,
		subdomains:['mt0','mt1','mt2','mt3']
});
var miniMap = new L.Control.MiniMap(miniMe, { toggleDisplay: true, zoomLevelFixed:11, /*autoToggleDisplay: true, */minimized: true }).addTo(map);

// buttons on map
L.easyButton('<img src="lib/zoom-out.svg" width="100%" title="zoom to see all stops in top table" data-toggle="tooltip" data-placement="right">', function(btn, map){

	if( routeTable.getDataCount(true) && mappedList.length )
		map.fitBounds(stopsLayer.getBounds(), {padding:[20,20], maxZoom:15});
}).addTo(map);

L.easyButton('<img src="lib/route.svg" width="100%" title="toggle route lines" data-toggle="tooltip" data-placement="right">', function(btn, map){
	routeLines();
}).addTo(map);

// ########################
// RUN ON PAGE LOAD
$(document).ready(function() {
	
	$(window).resize(function(){
		//$("#left-table").tabulator("redraw");
		//$("#right-table").tabulator("redraw");
		routeTable.redraw();
		databankTable.redraw();
	});
	
	hide( document.querySelector("#panel") );

	map.on('click', function(e) {
		map.panTo(e.latlng);
		dragmarker.setLatLng(e.latlng);
		if(!clickedflag) { 
			map.addLayer(dragmarker);
			clickedflag=true; 
			databankTable.deselectRow(); // deselect selected row if any to show that we are now in manual mode.
		}
		
		var selectedRows = routeTable.getSelectedData(); //$("#left-table").tabulator("getSelectedRows"); //get array of currently selected row components.
		if(selectedRows.length) {
			var row = selectedRows[0];
			var lat = Math.round(( e.latlng.lat + 0.0000001) * 100000) / 100000;
			var lon = Math.round(( e.latlng.lng + 0.0000001) * 100000) / 100000;
			show( document.querySelector("#panel") );
			$('#location').html(`${lat},${lon}<br><button class="btn btn-sm btn-danger" onclick="mapToSelectedRow(${lat},${lon})">Pin "${row.stop_name}" here</button>` );
		}
	});
		
	// URL parameters. from https://stackoverflow.com/a/2405540/4355695
	var query = window.location.search.substring(1).split("&");
	for (var i = 0, max = query.length; i < max; i++)
	{
		if (query[i] === "") // check for trailing & with no param
			continue;
		var param = query[i].split("=");
		URLParams[decodeURIComponent(param[0])] = decodeURIComponent(param[1] || "");
		// this gets stored to global json variable URLParams
	}

	loadRoutesList(); // defined in common.js

	let fileInput = document.getElementById('gpxFile');
	fileInput.addEventListener('change', function(e) {
		gpxUpload();
	});

	// populate confidence dropdown
	var confidenceCollector = '';
	Object.keys(confidence_options).forEach(element => {
		confidenceCollector += `<option value="${element}">${confidence_options[element]}</option>`;
	});
	$('#confidenceSelect').html(confidenceCollector);

});

//#####################################
// API Call functions

function loadRoute(firstTime=true) {
	
	if(firstTime) globalRoute = $("#routeSelect").val();
	
	console.log(globalRoute);

	$('#workingFileLoadStatus').html('loading..');
	$('#routeSaveStatus').html('');

	$.get( `${APIpath}loadJson?route=${globalRoute}`, function( response ) {
		var data = JSON.parse(response);
		//console.log(data);
		// get stopsArray0, stopsArray1
		for(i=0;i<data['stopsArray0'].length; i++) {
			data['stopsArray0'][i]['direction_id'] = 0;
			data['stopsArray0'][i]['stop_sequence'] = i+1;
		}

		for(i=0;i<data['stopsArray1'].length; i++) {
			data['stopsArray1'][i]['direction_id'] = 1;
			data['stopsArray1'][i]['stop_sequence'] = i+1;
		}
		

	
		var tableData = new Array();
		// to do : make child rows/tables for auto-suggest
		data['stopsArray0'].forEach(element => {
			tableData.push(makeStopRow(element));
		});
		data['stopsArray1'].forEach(element => {
			tableData.push(makeStopRow(element));
		});

		//console.log(tableData);
		routeTable.setData(tableData);
		$('#workingFileLoadStatus').html('');
		if (!map.hasLayer(stopsLayer)) map.addLayer(stopsLayer);
		mapStops();

	}).fail(function(err) {
		$('#workingFileLoadStatus').html(err.responseText);
	});

}



function saveRoute() {
	// globalRoute; // take the last loaded route value
	console.log('saveRoute(): filename is:',globalRoute);
	var sequence = routeTable.getData(); //$("#left-table").tabulator('getData');
	if(!sequence.length || !globalRoute) {
		$('#routeSaveStatus').html('Nothing to save.');
		return;
	}

	$('#routeSaveStatus').html('Saving...');
	
	pw = prompt('Please provide a Mapper level password.');

	$.ajax({
		url : `${APIpath}loadJson?route=${globalRoute}&pw=${pw}`,
		type : 'POST',
		data : JSON.stringify(sequence),
		cache: false,
		processData: false,  // tell jQuery not to process the data
		contentType: false,  // tell jQuery not to set contentType
		success : function(returndata) {
			console.log('Successfully sent data via POST to server /API/loadJson, resonse received: ' + returndata);
			$('#routeSaveStatus').html(`<p class="alert alert-success">${returndata}</p>`);
			setTimeout(function(){ $('#routeSaveStatus').html(''); }, 20000);
		},
		error: function(jqXHR, exception) {
			console.log( jqXHR.responseText );
			$('#routeSaveStatus').html(`<p class="alert alert-danger">${jqXHR.responseText}</p>`);
			setTimeout(function(){ $('#routeSaveStatus').html(''); }, 20000);
		}
	});

}

function gpxUpload() {
	var fileInput = document.getElementById('gpxFile');
	if (!fileInput.files || fileInput.files.length <= 0) {
		return;
	}
	
	var formData = new FormData();
	formData.append('gpx', fileInput.files[0]);

	$.ajax({
		url : `${APIpath}GPXUpload`,
		type : 'POST',
		data : formData,
		cache: false,
		processData: false,  // tell jQuery not to process the data
		contentType: false,  // tell jQuery not to set contentType
		success : function(returndata) {
			console.log('API/GPXUpload POST request with file upload successfully done.');
			mapGPX(JSON.parse(returndata));
		},
		error: function(jqXHR, exception) {
			console.log( jqXHR.responseText );
		}
	});
}

function routeSuggest() {
	// use globalRoute
	$.get( `${APIpath}routeSuggest?route=${globalRoute}`, function( response ) {
		$('#routeSuggestStatus').html(`<p class="alert alert-success">${response}</p>`);
		loadRoute(firstTime=false);

	}).fail(function(err) {
		$('#routeSuggestStatus').html(`<p class="alert alert-danger">${err.responseText}</p>`);
	});
}

function lockRoute() {
	// globalRoute
	// basic check : if all stops aren't mapped, Illa Po!
	routeTable.clearFilter(true);
	var data = routeTable.getData();
	mappedList = data.filter(a => ! isNaN(parseFloat(a.stop_lat))); // .filter() : from https://medium.com/poka-techblog/simplify-your-javascript-use-map-reduce-and-filter-bd02c593cc2d
	if( mappedList.length < data.length) {
		$('#routeLockStatus').html(`<p class="alert alert-danger">Map all the stops first!</p>`);
		setTimeout(function() { $('#routeLockStatus').html(''); },20000);
		return;
	}

	pw = prompt('Please provide an Admin level password.');
	$.get( `${APIpath}routeLock?route=${globalRoute}&pw=${pw}`, function( response ) {
		$('#routeLockStatus').html(`<p class="alert alert-success">${response}</p>`);
		setTimeout(function() { $('#routeLockStatus').html(''); },20000);

	}).fail(function(err) {
		$('#routeLockStatus').html(`<p class="alert alert-danger">${err.responseText}</p>`);
		setTimeout(function() { $('#routeLockStatus').html(''); },20000);
	});

}
// ########################################################
// JS FUNCTIONS

function makeStopRow(element) {
	let rowJson = { direction_id:element.direction_id, stop_sequence:element.stop_sequence, stop_name:element.stop_name};
	if(element.stop_lat) rowJson['stop_lat'] = parseFloat( String(element.stop_lat).trim() );
	if(element.stop_lon) rowJson['stop_lon'] = parseFloat( String(element.stop_lon).trim() );
	if(element.offset) rowJson['offset'] = element.offset;
	if(element.confidence) rowJson['confidence'] = element.confidence;
	if(element.stop_desc) rowJson['stop_desc'] = element.stop_desc;
	if(element.suggested) rowJson['suggested'] = element.suggested;
	return rowJson;
}


function loadDatabank(full=false) {
	// using papa.parse to load csv directly at client side instead of pestering the API.
	// since we're not doing fuzzy business anymore.
	Papa.parse("./databank/stops-databank.csv", {
		download: true,
		header: true,
		skipEmptyLines: true,
		complete: function(results, file) {
			//console.log("Parsing complete:", results, file);
			databankTable.setData(results.data);
		}
	});
}

function mapToSelectedRow(lat,lon) {
	console.log('mapToSelectedRow():',lat,lon);
	var selectedRows = routeTable.getSelectedRows(); //$("#left-table").tabulator("getSelectedRows");
	var row = selectedRows[0];
	// row component : http://tabulator.info/docs/3.5#component-row
	row.getCell('stop_lat').setValue(lat);
	row.getCell('stop_lon').setValue(lon);
	row.getCell('confidence').setValue($('#confidenceSelect').val());
	//row.getCell('source').setValue('manual');
	//row.getCell('source_stop_id').setValue('');
	//row.getCell('source_stop_name').setValue('');
	mapStops();	
}


// who's using this??
function updateLatLng(latlong, revflag) {
	if (revflag) {
		lat = parseFloat( latlong.split(',')[0] );
		lng = parseFloat( latlong.split(',')[1] );
		dragmarker.setLatLng([lat, lng]);
		map.panTo([lat, lng]);
	} else {
		lat = Math.round(( dragmarker.getLatLng().lat + 0.0000001) * 10000) / 10000;
		// Rounding, from https://stackoverflow.com/questions/11832914/round-to-at-most-2-decimal-places-only-if-necessary. The +0.000.. is to trip forward a number hinging on x.9999999...
		lng = Math.round(( dragmarker.getLatLng().lng + 0.0000001) * 10000) / 10000;
		document.getElementById('newlatlng').value = lat + ',' + lng;
		//document.getElementById('longitude').value = marker.getLatLng().lng;
		//map.panTo(dragmarker.getLatLng());
	}
}

function mapDataBank(limit=false, filter=true) {

	databank.clearLayers();
	//if (map.hasLayer(stopsLayer)) map.removeLayer(stopsLayer);

	if(filter)
		var stopsjson = databankTable.getData(true); //$("#right-table").tabulator("getData",true); // gets filtered data from table
	else
		var stopsjson = databankTable.getData(); //$("#right-table").tabulator("getData"); // gets full data from table

	if(debugMode) {
		console.log('mapDataBank: first stop:',stopsjson[0]);
		console.log('mapDataBank: limit:' + limit);
	}
	if(limit) {
		stopsjson = stopsjson.slice(0,limit);
	}
	for (i = 0; i < stopsjson.length; i++) {
		stoprow = stopsjson[i];
		// debugging: 

		let lat = parseFloat(stoprow['stop_lat']);
		let lon = parseFloat(stoprow['stop_lon']);
		if( ! checklatlng(lat,lon) ) {
			console.log('mapDataBank: Skipping a stop because of invalid values:', stopsjson[stoprow]);
			continue;
		}

		// stopRadius = 1 + 5*parseFloat(stoprow[param])/100;
		// no no, too complicated at frontend, don't bother grading the markers by the scores.

		var circleMarkerOptions = {
			renderer: myRenderer,
			radius: 4,
			fillColor: databankColor,
			color: 'black',
			weight: 0.5,
			opacity: 1,
			fillOpacity: 0.5
		};
		var id = stoprow.stop_id;
		if(id.length >= 10) {
			id = id.substr(0,3) + '..' + id.substr(id.length-5);
		}
		var tooltipContent = `${i+1}: ${stoprow.stop_name}<br>${stoprow.source}: ${id}`;

		var stopmarker = L.circleMarker([lat,lon], circleMarkerOptions)
			.bindTooltip(tooltipContent, {
				direction:'top', 
				offset: [0,-5]
			});
		stopmarker.properties = stoprow;
		stopmarker.addTo(databank);
	}
	
	if( ! map.hasLayer(databank) ) map.addLayer(databank);

	//map.fitBounds(databank.getBounds(), {padding:[-20,-20], maxZoom:15}); 

}


function mapStops(firstRun=false, zoomBack=true){

	if(firstRun) firstRunDone = true; // only dataloaded callback sends firstRun as true

	if(!firstRunDone) {
		if(debugMode) console.log('mapStops called but firstRunDone flag not set yet.');
		return;
	}

	stopsLayer.clearLayers();

	// calculation
	let totalRowsNum = routeTable.getDataCount();
	
	var data = routeTable.getData(true); //$("#left-table").tabulator('getData',true);
	mappedList = data.filter(a => ! isNaN(parseFloat(a.stop_lat)));
	// .filter() : from https://medium.com/poka-techblog/simplify-your-javascript-use-map-reduce-and-filter-bd02c593cc2d
	// and figured out a way to detect if a variable is a number and not null or undefined or blank or string
	percentage = Math.round( mappedList.length / totalRowsNum * 100 );
	$('#numMapped').html(`<b>${mappedList.length} (${percentage}%)</b> stops mapped so far.`);
	
	for (i = 0; i < mappedList.length; i++) {
		stoprow = mappedList[i];
		
		let lat = parseFloat(stoprow['stop_lat']);
		let lon = parseFloat(stoprow['stop_lon']);
		
		if( ! checklatlng(lat,lon) ) {
			console.log('Skipping a stop because of invalid values:', stopsjson[stoprow]);
			continue;
		}

		var circleMarkerOptions = {
			renderer: myRenderer,
			radius: 6,
			fillColor: directionDecide(stoprow),
			color: 'black',
			weight: 0.5,
			opacity: 1,
			fillOpacity: 0.7
		};
		var tooltipContent = `${stoprow.stop_name}`;
		if(stoprow.stop_sequence) tooltipContent = `${stoprow.stop_sequence}:${tooltipContent}`; // prefix sequence
		
		var stopmarker = L.circleMarker([lat,lon], circleMarkerOptions).bindTooltip(tooltipContent);
		stopmarker.properties = stoprow;
		stopmarker.addTo(stopsLayer);
	}
	if (map.hasLayer(stopsLayer) && mappedList.length & zoomBack) 
		//map.fitBounds(stopsLayer.getBounds(), {padding:[20,20], maxZoom:15}); 
		;
	// if the layer is activated, then zoom to fit.
}

function directionDecide(stoprow) {
	if( (stoprow.direction) && (stoprow.direction == '1') )  return stopReturnColor;
	else return stopOnwardColor;
}


function mapZoomHere(lat,lon,zoom=16) {
	if( ! checklatlng(lat,lon) ) {
		console.log('mapZoomHere(): invalid lat-lon values:', lat,lon);
		return;
	}
	map.flyTo([lat,lon], zoom, {duration:1, animate:true});
}

function seeGoogleMaps(){
	let lat = map.getCenter().lat;
	let lon = map.getCenter().lng;
	let zoom = map.getZoom();
	var url = `http://maps.google.com/maps?q=${lat},${lon}+(My+Point)&z=${zoom}&ll=${lat},${lon}`;
	// from https://webapps.stackexchange.com/a/54163/162017 
	console.log(url);
	var win = window.open(url, '_blank');
	}

function askToMapSelected(lat,lon, otherdata=''){
	if( ! checklatlng(lat,lon) ) {
		console.log('askToMapSelected: invalid lat-lon values:', lat,lon);
		return;
	}
	var selectedRows = routeTable.getSelectedData(); //$("#left-table").tabulator("getSelectedRows"); //get array of currently selected row components.
	if(selectedRows.length) {
		var row = selectedRows[0];
		
		$('#location').html(`${lat},${lon}, ${otherdata}<br><button class="btn btn-primary btn-sm" onclick="mapCopyOver()">Pin "${row.stop_name}" here</button>` );
		show( document.querySelector("#panel") );
	}
}

function mapCopyOver() {

	var selectedRows = routeTable.getSelectedRows(); //$("#left-table").tabulator("getSelectedRows"); //get array of currently selected row components.
	if(selectedRows.length) {
		var row = selectedRows[0];

		var rightSelected = databankTable.getSelectedData();// $("#right-table").tabulator("getSelectedRows")
		
		if(rightSelected.length) {
			var source = rightSelected[0];
			console.log('mapCopyOver: source:',source);
		
			row.getCell('stop_lat').setValue(source.stop_lat);
			row.getCell('stop_lon').setValue(source.stop_lon);
			//row.getCell('source').setValue(source.source);
			//row.getCell('source_stop_id').setValue(source.stop_id);
			//row.getCell('source_stop_name').setValue(source.stop_name);
			mapStops(firstRun=false, zoomBack=false);
		}	
		else {
			alert('Error: No row in data bank table is selected.');
			return;
		}
	}
	else {
		alert('Error: No row in stops table is selected.');
		return;
	}
}



function mapGPX(data) {
	console.log(data);

	trackLayer.clearLayers();
	var shapeLine = L.polyline.antPath(data['track'], {color: 'yellow', weight:3, interactive:false}).addTo(trackLayer);
	if (!map.hasLayer(trackLayer)) map.addLayer(trackLayer);

	waypointLayer.clearLayers();
	var circleMarkerOptions = {
			renderer: myRenderer,
			radius: 5,
			fillColor: 'orange',
			color: 'black',
			weight: 1,
			opacity: 1,
			fillOpacity: 0.7
	};
	for (i = 0; i < data.waypoint.length; i++) {
		row = data.waypoint[i];
		var tooltipContent = `GPX:${i+1}: <b>${row.name}</b><br>${row.time}`;
		let lat = parseFloat(row.lat);
		let lon = parseFloat(row.lon);
		var waypointMarker = L.circleMarker([lat,lon], circleMarkerOptions).bindTooltip(tooltipContent);
		waypointMarker.properties = row;
		waypointMarker.addTo(waypointLayer);
	}
	if (!map.hasLayer(waypointLayer)) map.addLayer(waypointLayer);
}

function addStop() {
	routeTable.addRow();
	//$("#left-table").tabulator('addRow');
}
function routeLines() {
	data = routeTable.getData(true); //$("#left-table").tabulator('getData',true);
	if(! data.length) return;
	
	if(lineLayer.getLayers().length && map.hasLayer(lineLayer)) {
		// toggle off if already loaded and visible
		map.removeLayer(lineLayer);
		return;
	}

	lineLayer.clearLayers();

	var dir0Line = [];
	var dir1Line = [];
	for( i=0; i<data.length; i++) {
		let lat = parseFloat(data[i].stop_lat);
		let lon = parseFloat(data[i].stop_lon);
		if(checklatlng(lat,lon)) {
			if (data[i].direction == '1' || data[i].direction == 1)
				dir1Line.push([lat,lon]);
			
			else dir0Line.push([lat,lon]);
		}
		
	}

	if(dir0Line.length)
		var dir0MapLine = L.polyline.antPath(dir0Line, {color: 'green', weight:2, delay:2000, interactive:false }).addTo(lineLayer);
	

	if(dir1Line.length)
		var dir1MapLine = L.polyline.antPath(dir1Line, {color: 'blue', weight:2, delay:2000, interactive:false }).addTo(lineLayer);

	
	if (!map.hasLayer(lineLayer))
		map.addLayer(lineLayer);
}

